#include "krb_locl.h"

RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/lib/krb/k_localtime.c,v 1.2 1995/10/24 10:14:30 bg Exp $");

#include <time.h>

struct tm *k_localtime(u_int32_t *tp)
{
  time_t t;
  t = *tp;
  return localtime(&t);
}
