/*
 * $Source: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/appl/sample/simple.h,v $
 * $Author: d91-jda $
 * $Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/appl/sample/simple.h,v 1.1.1.1 1995/10/23 11:17:42 d91-jda Exp $
 *
 * Copyright 1988 by the Massachusetts Institute of Technology.
 *
 * For copying and distribution information, please see the file
 * <mit-copyright.h>.
 *
 * Common definitions for the simple UDP-based Kerberos-mediated
 * server & client applications.
 */

#include <mit-copyright.h>

#define SERVICE	"sample"
#define	HOST	"bach"
