/*
 * $Source: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/lib/kdb/kdc.h,v $
 * $Author: bg $
 * $Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/lib/kdb/kdc.h,v 1.2 1995/08/10 22:08:56 bg Exp $
 *
 * Copyright 1987, 1988 by the Massachusetts Institute of Technology. 
 *
 * For copying and distribution information, please see the file
 * <mit-copyright.h>. 
 *
 * Include file for the Kerberos Key Distribution Center. 
 */

#ifndef KDC_DEFS
#define KDC_DEFS

#define S_AD_SZ		sizeof(struct sockaddr_in)

#define max(a,b)	(a>b ? a : b)
#define min(a,b)	(a<b ? a : b)

#define TRUE		1
#define FALSE		0

/* Don't depend on this! */
#ifndef MKEYFILE
#define MKEYFILE	"/.k"
#endif
#ifndef K_LOGFIL
#define K_LOGFIL	"/var/log/kpropd.log"
#endif

#define ONE_MINUTE	60
#define FIVE_MINUTES	(5 * ONE_MINUTE)
#define ONE_HOUR	(60 * ONE_MINUTE)
#define ONE_DAY		(24 * ONE_HOUR)
#define THREE_DAYS	(3 * ONE_DAY)

#endif /* KDC_DEFS */

