#ifndef __krb_locl_h
#define __krb_locl_h

#include "config.h"
#include "protos.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <assert.h>
#include <stdarg.h>

#include <errno.h>
#include <unistd.h>
#include <sys/types.h>

#if TIME_WITH_SYS_TIME
#include <sys/time.h>
#include <time.h>
#else
#if HAVE_SYS_TIME_H
#include <sys/time.h>
#else
#include <time.h>
#endif /* !HAVE_SYS_TIME_H */
#endif /* !TIME_WITH_SYS_TIME */

#include <sys/stat.h>
#include <fcntl.h>
#include <sys/file.h>
#ifdef HAVE_SYS_SELECT_H
#include <sys/select.h>
#endif
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include <krb.h>
#include <prot.h>

/* --- */

/* Globals! */
extern int krb_debug;
extern int krb_ap_req_debug;

/* Utils */
char *pkt_cipher __P((KTEXT));

int new_log __P((time_t, char *));

char *month_sname __P((int));
int fgetst __P((FILE *, char *, int));

#endif /*  __krb_locl_h */
