#include "bsd_locl.h"

RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/appl/bsd/getpass.c,v 1.2 1995/10/24 14:02:56 bg Exp $");

char *
getpass(const char *prompt)
{
  static char buf[128];
  des_read_pw_string(buf, sizeof(buf) - 1, prompt, 0);
  buf[sizeof(buf) - 1] = 0;
  return buf;
}
