#include "kdb_locl.h"

RCSID("$Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/lib/kdb/copykey.c,v 1.5 1995/10/24 16:27:59 bg Exp $");

void
copy_from_key(des_cblock in, u_int32_t *lo, u_int32_t *hi)
{
    bcopy(((char *) in) +           0, lo, sizeof(*lo));
    bcopy(((char *) in) + sizeof(*lo), hi, sizeof(*hi));
}

void
copy_to_key(u_int32_t *lo, u_int32_t *hi, des_cblock out)
{
    bcopy(lo, ((char *) out) +           0, sizeof(*lo));
    bcopy(hi, ((char *) out) + sizeof(*lo), sizeof(*hi));
}
