#ifndef __adm_locl_h
#define __adm_locl_h

#define TRUE 1
#define FALSE 0

#include "config.h"
#include "protos.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>

#include <sys/types.h>

#if TIME_WITH_SYS_TIME
#include <sys/time.h>
#include <time.h>
#else
#if HAVE_SYS_TIME_H
#include <sys/time.h>
#else
#include <time.h>
#endif /* !HAVE_SYS_TIME_H */
#endif /* !TIME_WITH_SYS_TIME */

#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <errno.h>

#include <netinet/in.h>

#include <des.h>
#include <krb.h>
#include <krb_db.h>
#include <kdc.h>

/* Utils */
long maketime __P((struct tm *, int));

#endif /*  __adm_locl_h */
