/*
 * $Source: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/kadmin/ksrvutil.h,v $
 *
 * $Locker:  $
 */


/*
 *
 * $Log: ksrvutil.h,v $
 * Revision 1.2  1995/09/28 15:01:05  bg
 * Assars changes.
 *
 *
 */


extern char *progname;
extern char u_name[], u_inst[], u_realm[];

void leave(char *str, int x);
void safe_read_stdin(char *prompt, char *buf, int size);
void safe_write(char *filename, int fd, char *buf, int len);

int yn(char *string);
int ny(char *string);

void ksrvutil_get(void);

