/*
 * Add here functions that don't have a prototype on your system.
 *
 * $Locker:  $
 *
 * $Header: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/include/protos.h,v 1.9 1995/10/23 12:02:40 bg Exp $
 *
 * $Log: protos.h,v $
 * Revision 1.9  1995/10/23 12:02:40  bg
 * Header!
 *
 *
 */


/* bcopy and friendes does not really exist but are used anyway! */
#include <memory.h>
#define bcopy(f,t,n)    memcpy((t), (f), (n))
#define bzero(s,n)      memset((s), 0, (n))
#define bcmp(a,b,n)     memcmp((a), (b), (n))

#if defined(__GNUC__) && defined(sun) && !defined(__svr4__) /* SunOS4 */

/* To get type fd_set */
#include <sys/types.h>
#include <sys/time.h>

/* To get struct sockaddr, struct in_addr and struct hostent */
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

/* To get struct stat */
#include <sys/stat.h>

/* To get struct utimbuf */
#include <utime.h>

int utime(const char *, struct utimbuf *);
int syscall(int, ...);
pid_t getpid(void);
int ftruncate(int, off_t);
int fchmod(int, mode_t);
int fsync(int);
int seteuid(uid_t);
int setreuid(int, int);
int flock(int, int);
int gettimeofday(struct timeval *tp, struct timezone *tzp);
int lstat(const char *, struct stat *);
int ioctl(int, int, void *); 
int gethostid(void);
int getpriority(int which, int who);
int setpriority(int which, int who, int priority);
int getdtablesize(void);
int initgroups(const char *name, int basegid);

int sigblock(int);
int sigsetmask(int);

int socket(int, int, int);
int setsockopt(int, int, int, void *, int);
int bind(int, void *, int);
int getsockname(int, struct sockaddr *, int *);
int accept(int, struct sockaddr *, int *);
int connect(int, struct sockaddr *, int);
int listen(int, int);
int recv(int s, void *buf, int len, int flags);
int recvfrom(int, char *, int, int, void *, int *);
int sendto(int, const char *, int, int, void *, int);
int select(int, fd_set *, fd_set *, fd_set *, struct timeval *);
int shutdown(int, int);
int getpeername(int, struct sockaddr *, int *);
int getsockopt(int, int, int, char *, int *);
int send(int s, const void *msg, int len, int flags);

char *inet_ntoa(struct in_addr in);
unsigned long inet_addr(const char *cp);
int gethostname(char *, int);
struct hostent *gethostbyname(const char *);

int strcasecmp(const char *, const char *);
void swab(const char *, char *, int);
int atoi(const char *str);
char *mktemp(char *);
void srandom(int seed);
int random(void);
int srand(unsigned int seed);
int rand(void);

int rcmd(char **, unsigned short, char *, char *, char *, int *);
int rresvport(int *);
int openlog(const char *ident, int logopt, int facility);
int syslog(int priority, const char *message, ...);
int ttyslot(void);

char *getpass(const char *);
char *crypt(const char *, const char *);

char *getusershell(void);
void setpwent();
void endpwent();

#endif /* SunOS4 */

#if defined(__GNUC__) && defined(sun) && defined(__svr4__) /* SunOS5 */

/* include this in a xxx_locl.h file #include <sys/filio.h> */

#endif

#if defined(__GNUC__) && defined(__osf__) /* OSF/1 */

/* To get type fd_set */
#include <sys/types.h>
#include <sys/time.h>

int select(int, fd_set *, fd_set *, fd_set *, struct timeval *);
int flock(int fildes, int operation);
int fsync(int fildes);
/* int setreuid(int ruid, int euid); */
int gethostname(char *address, int address_len);
int gethostid(void);
int ioctl(int d, unsigned long request, void * arg);

unsigned short htons(unsigned short hostshort);
unsigned int   htonl(unsigned int hostint);
unsigned short ntohs(unsigned short netshort);
unsigned int   ntohl(unsigned int netint);

char *mktemp(char *template);
char *getusershell(void);

#endif /* OSF/1 */

#if defined(__GNUC__) && defined(__sgi__) /* IRIX */

char *crypt(const char *key, const char *salt);

#endif /* IRIX */

#if defined(__GNUC__) && defined(_AIX) /* AIX */

char *crypt (char *PW, char *Salt);

struct timeval;
struct timezone;
int gettimeofday (struct timeval *Tp, struct timezone *Tzp);

#endif /* AIX */

#if defined(__GNUC__) && defined(__hpux) /* HP-UX */

int syscall(int, ...);

#endif /* HP-UX */
