/* 
 * Copyright 1987 by the Massachusetts Institute of Technology.
 *
 * For copying and distribution information,
 * please see the file <mit-copyright.h>.
 *
 * $Revision: 1.1.1.1 $
 * $Date: 1995/10/23 11:21:00 $
 * $State: Exp $
 * $Source: /afs/nada.kth.se/src/packages/kth-krb/SourceRepository/krb4/slave/kprop.h,v $
 * $Author: d91-jda $
 * $Locker:  $
 *
 * $Log: kprop.h,v $
 * Revision 1.1.1.1  1995/10/23 11:21:00  d91-jda
 * Imported sources
 *
 * Revision 1.1  1994/09/03 20:08:31  bg
 * Initial revision
 *
 * Revision 4.0  89/01/24  18:44:46  wesommer
 * Original version; programmer: wesommer
 * auditor: jon
 * 
 */

#define KPROP_SERVICE_NAME "rcmd"
#define KPROP_SRVTAB "/etc/srvtab"
#define TGT_SERVICE_NAME "krbtgt"
#define KPROP_PROT_VERSION_LEN 8
#define KPROP_PROT_VERSION "kprop01"
#define KPROP_TRANSFER_PRIVATE 1
#define KPROP_TRANSFER_SAFE 2
#define KPROP_TRANSFER_CLEAR 3
#define KPROP_BUFSIZ 32768
